@@ -44,6 +44,7 @@ def _exc_info_to_string(err, test):
     return ''.join(msgLines)
 
 class AutoStream(object):
+    """ File stream that creates the file on demand """
     def __init__(self, filename):
         self._filename = filename
         self._file = None
@@ -114,10 +115,11 @@ class NosePlugin(Plugin):
         self.status = "running"
         
         self.root_dir_name = time.strftime("%Y_%m_%d__%H_%M_%S")
+        self.root_dir_name = os.path.abspath(os.path.join(os.path.curdir, self.root_dir_name))
         os.mkdir(self.root_dir_name)
         shutil.copytree(resource_filename(__name__, "static"), os.path.join(self.root_dir_name, "static"))
         
-        self.result_html_path = os.path.abspath(os.path.join(os.path.curdir, self.root_dir_name, "result.html"))
+        self.result_html_path = os.path.join(self.root_dir_name, "result.html")
         self._ajax_server = AjaxServer(self._use_ajax, self._open_browser, self.result_html_path, 16193)
         self.create_html()
         self._ajax_server.trigget_start()
@@ -323,12 +325,20 @@ class NosePlugin(Plugin):
         self.html_log_link.attrib["href"] = self.log_name[-1]
         self.update_html()
          
-    def startX(self):
+    def startX(self, name):
+        self.name.append(name)
         for stream in self.log_stream:
             stream.flush()
         self.res.append("running")
         self.trace.append(None)
-        logname = os.path.join(*self.name) + ".txt"
+        logname = ''
+        log_trail = ''
+        for test_path_name in self.name:
+            logname = os.path.join(logname, test_path_name.replace(log_trail, ''))
+            log_trail = test_path_name + '.'
+        logname += ".txt"
+        for bad_char in "\\/:*?\"'<>|":
+            logname = logname.replace(bad_char, '')
         log_path = os.path.join(self.root_dir_name, logname)
         dirname = os.path.dirname(log_path)
         if dirname and not os.path.exists(dirname):
@@ -341,14 +351,19 @@ class NosePlugin(Plugin):
         self.stdouts.append(sys.stdout)
         sys.stdout = MultiStream([self.stdouts[0], self.log_stream[-1]])
         
-    def stopX(self):
+    def stopX(self, name):
         res = self.res.pop()
         if res == "running":
             res = "passed"
         trace = self.trace.pop()
         log_stream = self.log_stream.pop()
         self.log_handler.stream = self.log_stream[-1]
-        self.name.pop()
+        popped_name = self.name.pop()
+        # we don't get "stop" for some suites, so we need to make sure we're popping the name stack
+        # until we pop the name of the currently stopped context. we use endswith because for suites we
+        # add a prefix in the name stack which won't be in the "name" parameter
+        while not popped_name.endswith(name):
+            popped_name = self.name.pop()
         if trace is not None:
             log_stream.write(trace)
         sys.stdout = self.stdouts.pop()
@@ -362,15 +377,14 @@ class NosePlugin(Plugin):
         name = module.__name__
         desc = module.__doc__
         code = self.suite_code
-        self.name.append(name)
         self.module_failed_suites = 0
-        self.startX()
+        self.startX(name)
         self.log_stream[-1].add_callback(self.append_module)
         self.add_html_module(name, desc, code)
         self.html_module_span.attrib["class"] = "module_name module_" + self.res[-1]
         self.module_appended = False
     def stopModule(self, module):
-        res = self.stopX()
+        res = self.stopX(module.__name__)
         if res == "skipped":
             self.skipped_modules += 1
         elif self.module_failed_suites > 0:
@@ -382,14 +396,13 @@ class NosePlugin(Plugin):
         desc = suite.__doc__
         code = self.suite_code
         self.suite_failed_tests = 0
-        self.name.append(name)
-        self.startX()
+        self.startX(name)
         self.suite_appended = False
         self.log_stream[-1].add_callback(self.append_suite)
         self.add_html_suite(name, desc, code)
         self.html_suite_span.attrib["class"] = "suite_name suite_" + self.res[-1]
     def stopSuite(self, suite):
-        res = self.stopX()
+        res = self.stopX(suite.__name__)
         if res == "skipped":
             self.skipped_suites += 1
         elif self.suite_failed_tests > 0:
@@ -406,15 +419,14 @@ class NosePlugin(Plugin):
         code = self.suite_code
         self.append_module()
         self.append_suite()
-        self.name.append(name)
         self.total_tests += 1
         self.add_html_separator_hr(self.html_suite)
-        self.startX()
+        self.startX(name)
         self.add_html_test(name, desc, code)
         self.html_test_span.attrib["class"] = "test_name test_" + self.res[-1]
         self.update_html()
     def stopTest(self, test):
-        res = self.stopX()
+        res = self.stopX(test.id())
         if res == "skipped":
             self.skipped_tests += 1
         elif res == "failed":
